; Program MONITR31.ASM with DEF31.ASM and DECLARES.ASM
; A low-level monitor for the 8051 for the TASM assembler
; Copyright 1989 Ron Stubbers 
; This program is provided for personal, private use only.
; Commercial use in whole or in part without express written permission of
; the author is prohibited.  
; No warranty is made as to the suitability of the program for any 
; particular use.  
;
;_______________________________________________________________________
; REVISION HISTORY
;3/10/90 Robert Gallahan, 11300 Melody Dr.#307, Northglenn CO 80234
; 1.1	Syntax modified for PSEUDOSAM ASSEMBLER, include files not required.
;	Clean up user display, add interrupt support, ASCII dump display,
;	register & internal memory dump
;_______________________________________________________________________
;
	.equ	VERSION,'1'
	.equ	REVISION,'1'
	.equ	RAMBASE,h'2000		;beginning of RAM area
	.equ	STKBASE,H'40		;bottom of STACK area
	.EQU	ESC,h'1B		;escape character
;
	.ORG	RAMBASE
PATCH:	.rs	3		;reserve 3 bytes
				;SET UP LABEL ADDRESS FOR PATCH JUMP
;
	.ORG 	0		;THEN SET REST OF PROG AT 00H
;0000 is the processor's reset vector
         ajmp 	INIT
;
;Interrupt Service Routines
;0003 external INT0 (p3.2)
	.org 3
	ljmp	RAMBASE+3
;
;000B timer 0 
	.org	h'000b
	ljmp	RAMBASE+h'0b
;
;0013 external INT1 (p3.3)
	.org	h'0013
	ljmp	RAMBASE+h'13
;
;001b timer 1
	.org	h'001b
	ljmp	RAMBASE+h'1b
;
;0023 internal serial interface RI and TI
	.org	h'0023
	ljmp	RAMBASE+h'23
;------------------------------------------------------------------------
;		***** MAINLINE CODE STARTS HERE *****
;------------------------------------------------------------------------
INIT:	mov	a,#STKBASE		;set stack pointer
	mov	sp,a
	CLR	P1.5	;*** SHUT OFF BELL FOR WYSE 50
	clr	a
	mov	psw,a			;set register bank zero
;
;initalize interrupt vectors in RAM 
	mov	a,#h'32			;"RETI"  instruction
	mov	dptr,#RAMBASE+h'03	;INT0 vector in RAM
	movx	@dptr,a
	mov	dptr,#RAMBASE+h'0B	;timer 0 vector in RAM
	movx	@dptr,a
	mov	dptr,#RAMBASE+h'13	;INT1 vector in RAM
	movx	@dptr,a
	mov	dptr,#RAMBASE+h'1b	;timer 1 vector in RAM
	movx	@dptr,a
	mov	dptr,#RAMBASE+h'23	;serial RI/TI vector in RAM
	movx	@dptr,a
;
;initialize CPU's serial port
	MOV	SCON,#h'52
	MOV	TMOD,#h'20
	MOV	TH1,#h'FD 	;#0E8H=1200 BAUD  #0FDH=9600 BAUD
	SETB	TR1		;START TIMER FOR SERIAL PORT
;
MENU:	LCALL 	XSTRING
	.db	"\n\r"
	.db	"MONITR31       RTC-31 System Monitor     VERSION "
	.db	VERSION,'.',REVISION,"\r\n\n"
	.db	"ENTER THE LETTER FOR YOUR SELECTION\n\r\n\r"
	.db	"D - DUMP External RAM\n\r"
	.db	"R - READ Internal RAM (indirect)\n\r"
	.db	"L - LOAD External RAM WITH BYTE\n\r"
	.db	"J - JUMP to Address\n\r"
	.db	"P - DOWNLOAD Hex File to RAM\n\r"
	.db	"I - DIRECT READ Internal RAM  (0-7F & SFR's)\n\r"
	.db	"O - DIRECT WRITE Internal RAM (0-7F & SFR's)\n\r"
	.db	"X - Display REGISTER banks & internal ram\n\r"
	.db	"\n\n\n\n\n\n\n\n\n\n"	;advance to 24th line
	.db	esc
getsel:	lcall	xstring
	.db	"\n:"		;prompt
	.DB	ESC

	LCALL 	CHR_IN
	anl	a,#h'5f		;upcase, strip parity

D_TEST:	CJNE	A,#'D',R_TEST	;IF NOT D, THEN CHECK R
	AJMP	D_RUN		;D, SO RUN DUMP MEMORY
R_TEST:	CJNE	A,#'R',L_TEST	;IF NOT R, THEN CHECK L
	AJMP	R_RUN		;R, SO RUN REGISTER SET/READ
L_TEST:	CJNE	A,#'L',J_TEST	;IF NOT L, THEN CHECK J
	AJMP	L_RUN		;L, SO RUN LOAD RAM W/ BYTE
J_TEST:	CJNE	A,#'J',P_TEST	;IF NOT J, THEN CHECK P
	AJMP	J_RUN		;J, SO RUN JUMP TO ADDRESS
P_TEST:	CJNE	A,#'P',I_TEST	;IF NOT P, THEN CHECK I
	AJMP	P_RUN		;P, SO RUN DOWNLOAD PROGRAM
I_TEST:	CJNE	A,#'I',O_TEST	;IF NOT I, THEN NO DIRECT READ
	AJMP	I_RUN		;I, SO DIRECT READ
O_TEST:	CJNE	A,#'O',X_test	;IF NOT O, THEN CHECK for X
	AJMP	O_RUN		;O, SO DIRECT WRITE
X_test:	cjne	a,#'X',J_MENU	;go if no match
	ajmp	disp_reg	;yep - go dump registers
J_MENU: ACALL 	WAIT		;default - wait, then show menu
	LJMP	MENU

;------------------------------------------------------------------------
;Display external data memory (hex and ASCII)
D_RUN:	LCALL	XSTRING		;ASK FOR START ADDRESS
	.db	"\n\rENTER 2-BYTE START ADDRESS \n\r\n\r"
	.DB	ESC
	ACALL	READHEX		;PUT START ADDR IN DPTR
	MOV	DPH,A
	ACALL	READHEX
	ANL	A,#h'F0		;START ON 16-BYTE BOUNDARY
	MOV	DPL,A		;HAVE START ADDRESS IN DPTR
	PUSH	DPH		;save start address on stack
	PUSH	DPL
	LCALL	XSTRING
	.db	"\n\rHOW MANY 16-BYTE ROWS (00 TO FF)\n\r\n\r"
	.DB	ESC
	POP	DPL
	POP	DPH		;restore start address to DPTR
	ACALL	READHEX
	MOV	R1,A		;SET UP ROW COUNTER
	ACALL	LFCR		;GO TO NEXT LINE TO START
	INC	R1		;MAKE SURE AT LEAST ONE
LOUTLOOP:
	MOV	R0,#16		;SET UP BYTE COUNTER
	MOV	A,DPH		;AT START OF LINE OUTPUT ADDR
	ACALL	WRBYTE		;
	MOV	A,DPL		;
	ACALL	WRBYTE		;
	MOV	A,#' '		;AND SPACE TO MAKE IT TIDY
	ACALL	CHR_OUT		;
	push	dpl
	push	dph		;save line start address on stack
BOUTLOOP:
	MOVX	A,@DPTR		;OUTPUT ROW ONE BYTE AT A TIME
	ACALL	WRBYTE
	MOV	A,#' '
	ACALL	CHR_OUT
	INC	DPTR
	DJNZ	R0,BOUTLOOP	;END OF BYTE LOOP
	pop	dph
	pop	dpl		;restore line start address
	acall	dascii		;display ascii equivalents
	ACALL	LFCR		;GO TO NEXT LINE
	DJNZ	R1,LOUTLOOP	;LOOP THROUGH NEXT LINE IF REQ'D
	acall	lfcr		;drop down a line
	LJMP	getsel		;and return to command mode
;
dascii: mov	r0,#16		;initalize loop count
	mov	a,#' '
	acall	chr_out		;put out couple of spaces
	acall	chr_out
dasci1:	movx	a,@dptr		;get the byte
	anl	a,#h'7f		;mask parity bit
	clr	c		;clear carry
	subb	a,#' '		;subtract ascii space
	jnc	dasci2		;jump if displayable
	mov	a,#('.'-' ')	;load "." minus space if not displayable
dasci2:	add	a,#' '		;make it ascii again
	acall	chr_out		;print it
	inc	dptr		;bump data pointer
	djnz	r0,dasci1	;and loop 'till done
	ret			;return and do next line
;
;------------------------------------------------------------------------
R_RUN:	ACALL	XSTRING
	.db	"\n\rENTER MEMORY BYTE ADDRESS (IN HEX, BOTH NIBBLES) "
	.db	"\n\r(Note that SFRs cannot"
	.db	" be read as an indirect register move is used.)\n\r"
	.DB	ESC
	ACALL	READHEX		;GET ADDRESS BYTE 
	MOV	R0,A		;PUT THE ADDRESS INTO A REGISTER
	MOV	A,#' '		;PUT A SPACE BETWEEN REG ADDR AND DATA
	ACALL	CHR_OUT
	MOV	A,@R0		;USE REGISTER INDIRECT TO GET DATA
	ACALL	WRBYTE		;AND SEND IT TO THE CONSOLE
	ACALL	LFCR
	LJMP	getsel
;
;------------------------------------------------------------------------
L_RUN:	ACALL	XSTRING
	.db	"\n\rENTER THE 2-BYTE STARTING ADDRESS YOU WISH TO CHANGE"
	.db	"\n\rNOTE THAT WRITING TO ROM WILL NOT CAUSE AN ERROR\n\r"
	.DB	ESC
	ACALL	READHEX		;FIRST GET THE TWO ADDRESS BYTES
	MOV	DPH,A
	ACALL	READHEX
	MOV	DPL,A
	PUSH	DPH		;SAVE THE ADDRESS
	PUSH	DPL
	MOV	A,#' '		;SPACE BETWEEN ADDRESS AND DATA
	ACALL	CHR_OUT
	MOVX	A,@DPTR		;SHOW THE CURRENT DATA
	ACALL	WRBYTE
	ACALL	XSTRING
	.db	"\n\rENTER THE BYTE TO BE WRITTEN (IN HEX, BOTH NIBBLES)\n\r"
	.DB	ESC
	ACALL	READHEX		;GET THE NEW VALUE
	POP	DPL
	POP	DPH
	MOVX	@DPTR,A		;MOVE THE NEW VALUE TO MEMORY
	LJMP	getsel		;return to command mode
;
;------------------------------------------------------------------------
;Jump to address 
J_RUN:	ACALL	XSTRING
	.db	"\n\rENTER THE 2-BYTE JUMP DESTINATION ADDRESS\n\r"
	.DB	ESC
	ACALL	READHEX		;FIRST GET THE JUMP ADDRESS
	MOV	DPH,A
	ACALL	READHEX
	MOV	DPL,A
	clr	a
;	MOV	A,#h'00
	JMP	@A+DPTR		;JUMP TO THE ADDRESS
;
;------------------------------------------------------------------------
;Load hex file into memory
P_RUN:	ACALL	LFCR
	ACALL	LFCR
	ACALL	CHR_IN		;GET RID OF : AT START OF LINE
	ACALL	READHEX
	JZ	ALLREAD		;LAST RECORD HAS 01 IN RECORD TYPE
	MOV	R5,A		;LOAD BYTE COUNT FOR LINE INTO R5
	ACALL	READHEX		;READ HIGH BYTE OF START ADDRESS
	MOV	DPH,A		
	ACALL	READHEX		;READ LOW BYTE OF START ADDRESS
	MOV	DPL,A
	ACALL	READHEX		;READ RECORD TYPE 00 = DATA, 01 = END
READLOOP:
	ACALL	READHEX
	MOVX	@DPTR,A
	INC	DPTR		;MOVE DPTR TO NEXT BYTE ADDRESS
	DJNZ	R5,READLOOP	;DECR BYTE COUNTER, JNZ
	ACALL	READHEX		;GET RID OF CHECKSUM, NO CHECKING
	ACALL	CHR_IN		;GET RID OF LINE FEED
	ACALL	CHR_IN		;GET RID OF CARRIAGE RETURN
	AJMP	P_RUN		;GET NEXT LINE
ALLREAD:
	ACALL	READHEX		;READ ADDRESS
	ACALL	READHEX		;READ ADDRESS
	ACALL	READHEX		;READ RECORD TYPE
	ACALL	READHEX		;READ LAST CHECKSUM
	ACALL	CHR_IN		;READ LAST CR
	ACALL	CHR_IN 		;READ LAST LF
	LJMP	getsel		;DONE, GO BACK TO MENU
;
;------------------------------------------------------------------------
;Read internal RAM using direct addressing 
I_RUN:	acall	sfrlist		;display SFR crib sheet
	MOV	dptr,#PATCH	;MOV INSTRUCTION WILL BE ASSEMBLED
	MOV	A,#h'0E5		;E5 IS A MOV A,DIRECT
	MOVX	@DPTR,A		;PUT THE INSTRUCTION PATCH
	INC	DPTR		;SET DPTR TO NEXT LOCATION (PATCH+1)
	PUSH 	DPH		;SAVE DPTR
	PUSH	DPL
	ACALL	XSTRING		;GET REGISTER ADDR TO READ
	.db	"\n\rENTER HEX ADDRESS OF DIRECT BYTE TO READ\n\r"
	.DB	ESC
	ACALL	READHEX
	POP	DPL		;GET PATCH DPTR VALUE BACK AFTER XSTRNG
	POP	DPH
	MOVX	@DPTR,A		;SEND ADDR OF BYTE TO COMPLETE INSTR.
	INC	DPTR		;
	MOV	A,#h'22		;PUT RET INSTRUCTION IN PATCH+2.
	MOVX	@DPTR,A		;
	LCALL	PATCH		;EXECUTE PATCH
	ACALL	WRBYTE		;SHOW BYTE RETRIEVED BY PATCH
	ACALL	LFCR
	LJMP	getsel
;
;------------------------------------------------------------------------
;Write internal RAM using direct addressing
O_RUN:	acall	sfrlist
	MOV	DPtr,#PATCH	;MOV INSTRUCTION WILL BE ASSEMBLED 
	MOV	A,#h'0F5		;F5 IS A MOV DIRECT,A
	MOVX	@DPTR,A		;PUT THE INSTRUCTION AT PATCH
	INC	DPTR		;SET DPTR TO NEXT LOCATION (PATCH+1)
	PUSH	DPH		;SAVE DPTR
	PUSH	DPL		
	ACALL	XSTRING		;GET REGISTER ADDR TO WRITE
	.db	"\n\rENTER HEX ADDRESS OF DIRECT BYTE TO WRITE\n\r"
	.DB	ESC
	ACALL	READHEX
	POP	DPL		;GET PATCH DPTR VALUE BACK AFTER XSTRNG
	POP	DPH
	MOVX	@DPTR,A		;SEND ADDR OF BYTE TO COMPLETE INSTR.
	INC	DPTR		;DPTR TO PATCH+2 FOR RET INSTR.
	MOV	A,#h'22
	MOVX	@DPTR,A		;PUT RET INSTRUCTION IN PATCH
	ACALL	XSTRING		;GET BYTE TO WRITE 
	.db	"\n\rENTER BYTE (IN HEX, BOTH NIBBLES) TO WRITE "
	.db	"TO DIRECT MEMORY\n\r"
	.DB	ESC
	ACALL	READHEX		;get value from user
	LCALL	PATCH		;DATA IN A, INSTRs AT PATCH, SO EXECUTE
	LJMP	getsel
;
;------------------------------------------------------------------------
;display registers
disp_reg:
	acall	xstring
	.db	"\n\rRegisters:\n\rBank  r0 r1 r2 r3 r4 r5 r6 r7\n\r"
	.db	esc
	clr	a
	mov	r0,a		;data pointer
dr1:	mov	a,r0
	rr	a
	rr	a
	rr	a		;make value a bank address
	acall	wrbyte		;display bank number
	mov	a,#' '
	acall	chr_out
	acall	chr_out
	acall	chr_out
	acall	chr_out		;output four spaces
dr2:	mov	a,@r0
	acall	wrbyte		;display the value
	mov	a,#' '
	acall	chr_out		;space between each one
	inc	r0		;bump pointer
	mov	a,r0		;get address
	anl	a,#h'07
	cjne	a,#0,dr2	;continue till bits 2-0 = zero
;
	acall	lfcr		;bump display to next line
	cjne	r0,#h'20,dr1	;continue 'till all four banks displayed
;
	acall	xstring
	.db	"\n\rInternal data ram:\n\r"
	.db	esc
dr3:	mov	a,r0
	acall	wrbyte		;display the address
	mov	a,#' '
	acall	chr_out
	acall	chr_out
	acall	chr_out
	acall	chr_out		;output four spaces
dr4:	mov	a,@r0
	inc	r0
	acall	wrbyte		;display the data
	mov	a,#' '
	acall	chr_out		;keep it pretty
	mov	a,r0
	anl	a,#h'0f
	cjne	a,#0,dr4	;16 bytes at a time
;
	acall	lfcr		;bump down to next line
	cjne	r0,#0,dr3	;loop 'till r3 overflows	
	ajmp	getsel		;and return to command mode
;------------------------------------------------------------------------
;display SFR crib sheet 
sfrlist:
	acall	xstring
	.db	"\n\rSpecial Function Register Addresses:\n\r"
	.db "\n\rP0    80   P1    90   P2   A0   P3   B0"
	.db "\n\rACC   E0   PSW   D0   B    F0   DPL  82   DPH  83   SP  81" 
	.db "\n\rTCON  88   TMOD  89   TL0  8A   TH0  8C   TL1  8B   TH1  8D"
	.db "\n\rSCON  98   SBUF  99   IE   A8   PCON 87\n\r"
	.db	esc
	ret
;------------------------------------------------------------------------
; LFCR SENDS A LINE FEED AND CARRIAGE RETURN OUT THE SERIAL PORT
; NOTHING IS PASSED IN, AND THE CONTENTS OF A ARE DESTROYED
;
LFCR:	MOV	A,#'\n'
	ACALL	CHR_OUT
	MOV	A,#'\r'
	ACALL	CHR_OUT
	RET
;
; WAIT PAUSES FOR A FRACTION OF A SECOND.  FREQUENCY UNKNOWN
;
WAIT:	MOV	A,#-1
WAIT1:	MOV	B,#-1
	DJNZ	B,*
	DEC	A
	JNZ	WAIT1
	RET	
;
; XSTRING IS FROM THE AMD 8051 DATA BOOK AND SENDS THE TEXT STRING
; THAT FOLLOWS THE CALL TO THE SERIAL PORT.  IT MESSES UP THE 
; DPTR AND ACC, AND ALTHOUGH IS NOT ENDED WITH A RET, MUST BE 
; CALLED.  THE TEXT STRING MUST END WITH AN ESC CHARACTER
;
XSTRING:
	POP	DPH		;LOAD DPTR WITH FIRST CHAR
	POP	DPL		
XSTR_1:	CLR	A		;(ZERO OFFSET)
	MOVC	A,@A+DPTR		;FETCH FIRST CHAR IN STRING
XSTR_2:	JNB	TI,*		;WAIT UNTIL TRANSMITTER READY
	CLR	TI		;MARK AS NOT READY
	MOV	SBUF,A		;OUTPUT NEXT CHARACTER
	INC	DPTR		;BUMP POINTER
	CLR	A		
	MOVC	A,@A+DPTR		;GET NEXT CHARACTER
	CJNE	A,#ESC,XSTR_2	;LOOP UNTIL ESCAPE READ
	MOV	A,#1
	JMP	@A+DPTR		;RETURN TO CODE AFTER ESCAPE
; 
; READS A CHARACTER INTO A FROM THE SERIAL PORT AND
; ECHOS THE CHARACTER BACK TO THE SERIAL PORT
;
CHR_IN:	JNB	RI,*
	CLR	RI
	MOV	A,SBUF
	ANL	A,#h'7F
	ACALL	CHR_OUT
	RET
;
; SENDS A CHARACTER IN A OUT THE SERIAL PORT.  MUST BE AN ASCII CHARACTER
;
CHR_OUT:
	JNB	TI,*
	CLR	TI
	MOV	SBUF,A
	RET
;
; CONVERTS AN ASCII CHARACTER IN A TO ITS HEX EQUIVALENT.
; RETURNS VALUE IN LOWER NIBBLE, UPPER NIBBLE ZEROS
;
ASC2HEX:
	CLR	CY
	SUBB	A,#'0'
	MOV	B,A
	SUBB	A,#10		;subtract 10 decimal
	JB	CY,A2LT10
	MOV	A,B
	SUBB	A,#7
	MOV	B,A
A2LT10:	MOV	A,B
	RET
;
; CONVERTS THE LOWER NIBBLE IN A TO AN ASCII CHARACTER RETURNED IN A
; 
HEX2ASC:
	ANL	A,#h'0F
	CLR	CY
	MOV	B,A
	SUBB	A,#10
	MOV	A,B
	JB	CY,H2LT10
	ADD	A,#7
H2LT10:	ADD	A,#'0'
	RET
;
; READS TWO ASCII CHARACTERS FROM THE SERIAL PORT AND 
; CONVERTS THEM INTO A BYTE RETURNED IN A.  USES
; REGISTER 2 IN THE FIRST BANK
;
READHEX:
	ACALL	CHR_IN		;GET THE FIRST CHARACTER
	ACALL	ASC2HEX		;CONVERT TO HEX NIBBLE
	SWAP	A		;HIGH ORDER NIBBLE IS FIRST ONE
	MOV	R2,A		;SAVE THE HIGH ORDER NIBBLE
	ACALL	CHR_IN		;GET LOW ORDER CHARACTER
	ACALL	ASC2HEX		;CHANGE TO HEX NIBBLE
	ORL	A,R2		;PUT NIBBLES TOGETHER IN A
	RET
;
; WRITES BYTE IN A TO THE SERIAL PORT AS TWO ASCII 
; CHARACTERS.  USES REGISTER 2 IN THE FIRST BANK.
;
WRBYTE:	MOV	R2,A
	SWAP	A
	ACALL	HEX2ASC
	ACALL	CHR_OUT
	MOV	A,R2
	ACALL	HEX2ASC
	ACALL	CHR_OUT
	RET
;
	.END
